@@ -1,5 +1,5 @@
 /***************************************************************************
- *   2007-2019 by Peter Semiletov <peter.semiletov@gmail.com>                            *
+ *   2007-2022 by Peter Semiletov <peter.semiletov@gmail.com>                            *
  *                                                                         *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -37,6 +37,7 @@ code from qwriter:
 #define DOCUMENT_H
 
 #include <vector>
+#include <utility>
 
 #include <QPoint>
 #include <QStatusBar>
@@ -44,9 +45,12 @@ code from qwriter:
 #include <QTabWidget>
 #include <QPlainTextEdit>
 #include <QSyntaxHighlighter>
+#include <QMultiHash> 
 
 #if QT_VERSION >= 0x050000
 #include <QRegularExpression>
+#else
+#include <QRegExp>
 #endif
 
 #if defined (JOYSTICK_SUPPORTED)
@@ -58,95 +62,12 @@ code from qwriter:
 #include "todo.h"
 
 
+using namespace std;
+
+
 class CDox;
 class CDocument;
-
-
-class CTEAEdit: public QPlainTextEdit
-{
-  Q_OBJECT
-
-public:
-
-  bool highlight_current_line;
-  bool hl_brackets;
-  bool draw_margin;
-  bool draw_linenums;
-  bool auto_indent;
-  bool spaces_instead_of_tabs;
-  int tab_sp_width; //in spaces
-  int brace_width; //in pixels
-  int margin_pos; //in chars
-  int margin_x;  //in pixels
-
-  QString indent_val;
-  QList <QTextEdit::ExtraSelection> extraSelections;
-  QTextEdit::ExtraSelection brace_selection;
-
-  QPoint rect_sel_start; //rect selection
-  QPoint rect_sel_end;   //rect selection
-
-  CDocument *doc; //uplink
-
-  QColor current_line_color;
-  QColor brackets_color;
-  QColor margin_color;
-  QColor linenums_bg;
-  QColor text_color;
-
-  QColor sel_text_color;
-  QColor sel_back_color;
-
-  QWidget *lineNumberArea;
-
-
-  CTEAEdit (QWidget *parent = 0);
-
-  void set_show_linenums (bool enable);
-  void set_show_margin (bool enable);
-  void set_margin_pos (int mp);
-  void set_hl_cur_line (bool enable);
-  void set_hl_brackets (bool enable);
-
-  void set_word_wrap (bool wrap);
-  bool get_word_wrap();
-
-  void indent();
-  void un_indent();
-
-  void calc_auto_indent();
-  void setup_brace_width();
-  void braceHighlight();
-
-  void lineNumberAreaPaintEvent(QPaintEvent *event);
-  int lineNumberAreaWidth();
-
-  void text_replace (const QString &s);
-  void rect_sel_reset();
-  void rect_sel_replace (const QString &s, bool insert = false);
-  void update_ext_selections();
-  void update_rect_sel();
-  QString get_rect_sel();
-  void rect_sel_cut (bool just_del = false);
-  Q_INVOKABLE bool has_rect_selection();
-
-
-protected:
-
-  QMimeData* createMimeDataFromSelection();
-  bool canInsertFromMimeData (const QMimeData *source);
-  void insertFromMimeData (const QMimeData *source);
-  void paintEvent(QPaintEvent *event);
-  void keyPressEvent (QKeyEvent *event);
-  void resizeEvent(QResizeEvent *event);
-
-public slots:
-
-  void updateLineNumberAreaWidth (int newBlockCount);
-  void cb_cursorPositionChanged();
-  void updateLineNumberArea (const QRect &, int);
-  void slot_selectionChanged();
-};
+class CLineNumberArea;
 
 
 class CSyntaxHighlighter: public QSyntaxHighlighter
@@ -155,20 +76,17 @@ public:
 
   CDocument *document;
   bool casecare;
-  int xml_format;
 
-  QString exts;
-  QString langs;
-  QString cm_mult;
-  QString cm_single;
+  QString comment_mult;
+  QString comment_single;
 
-  QTextCharFormat singleLineCommentFormat;
-  QTextCharFormat multiLineCommentFormat;
+  QTextCharFormat fmt_multi_line_comment;
 
   CSyntaxHighlighter (QTextDocument *parent = 0, CDocument *doc = 0, const QString &fname = "none");
 };
 
 
+#if QT_VERSION < 0x050000
 class CSyntaxHighlighterQRegExp: public CSyntaxHighlighter
 {
   Q_OBJECT
@@ -179,22 +97,19 @@ protected:
 
 public:
 
-  struct HighlightingRule
-         {
-          QRegExp pattern;
-          QTextCharFormat format;
-         };
+  vector <pair <QRegExp, QTextCharFormat> > hl_rules;
 
-  QRegExp commentStartExpression;
-  QRegExp commentEndExpression;
+  pair <QRegExp, bool> comment_start_expr;
+  pair <QRegExp, bool> comment_end_expr;
 
   Qt::CaseSensitivity cs;
 
-  std::vector <HighlightingRule> highlightingRules;
-
   CSyntaxHighlighterQRegExp (QTextDocument *parent = 0, CDocument *doc = 0, const QString &fname = "none");
-  void load_from_xml (const QString &fname);
+  //void load_from_xml (const QString &fname);
+  void load_from_xml_pugi (const QString &fname);
+
 };
+#endif
 
 
 #if QT_VERSION >= 0x050000
@@ -208,74 +123,165 @@ protected:
 
 public:
 
-  struct HighlightingRule
-        {
-         QRegularExpression pattern;
-         QTextCharFormat format;
-        };
+  vector <pair <QRegularExpression, QTextCharFormat> > hl_rules;
 
-  QRegularExpression commentStartExpression;
-  QRegularExpression commentEndExpression;
   QRegularExpression::PatternOptions pattern_opts;
 
-  QTextCharFormat singleLineCommentFormat;
-  QTextCharFormat multiLineCommentFormat;
-
-  std::vector <HighlightingRule> highlightingRules;
+  pair <QRegularExpression, bool> comment_start_expr;
+  pair <QRegularExpression, bool> comment_end_expr;
 
   CSyntaxHighlighterQRegularExpression (QTextDocument *parent = 0, CDocument *doc = 0, const QString &fname = "none");
-  void load_from_xml (const QString &fname);
-};
+  //void load_from_xml (const QString &fname);
+  void load_from_xml_pugi (const QString &fname);
 
+};
 #endif
 
 
-class CDocument: public QObject
+class CDocument: public QPlainTextEdit
 {
   Q_OBJECT
 
+private:
+
+  CLineNumberArea *line_num_area;
+  QList <QTextEdit::ExtraSelection> extra_selections;
+  QTextEdit::ExtraSelection brace_selection;
+
+protected:
+
+  QAction *actCopy;
+  QAction *actCut;
+  QAction *actPaste;
+
+  QMimeData* createMimeDataFromSelection() const;
+  bool canInsertFromMimeData (const QMimeData *source) const;
+  void insertFromMimeData (const QMimeData *source);
+  void paintEvent (QPaintEvent *event);
+  void keyPressEvent (QKeyEvent *event);
+  void resizeEvent (QResizeEvent *event);
+  void wheelEvent (QWheelEvent *e);
+  void contextMenuEvent (QContextMenuEvent *event);
+
+
+  //QMenu* createStandardContextMenu();
+
 public:
 
-  bool cursor_xy_visible;
+  CDox *holder; //uplink
+  QWidget *tab_page; //pointer
 
-  QString eol;
-
-  CDox *holder;
   CSyntaxHighlighter *highlighter;
-  CTEAEdit *textEdit;
 
   QStringList labels;
 
+  QString eol;
   QString markup_mode;
   QString file_name;
   QString text_to_search;
   QString charset;
-  QWidget *tab_page;
-  int position;
+  QString indent_val;
 
-  CDocument (QObject *parent = 0);
+  QPoint rect_sel_start; //rect selection
+  QPoint rect_sel_end;   //rect selection
+
+  QColor current_line_color;
+  QColor brackets_color;
+  QColor margin_color;
+  QColor linenums_bg;
+  QColor text_color;
+  QColor sel_text_color;
+  QColor sel_back_color;
+
+  bool cursor_xy_visible;
+  bool highlight_current_line;
+  bool hl_brackets;
+  bool draw_margin;
+  bool draw_linenums;
+  bool auto_indent;
+  bool spaces_instead_of_tabs;
+  bool show_tabs_and_spaces;
+
+  int position;
+  int tab_sp_width; //in spaces
+  int brace_width; //in pixels
+  int margin_pos; //in chars
+  int margin_x;  //in pixels
+
+  CDocument (CDox *hldr, QWidget *parent = 0);
   ~CDocument();
 
-  Q_INVOKABLE QString get_selected_text() const;
-  Q_INVOKABLE void set_selected_text (const QString &value);
+  QString get() const; //return selected text
+  void put (const QString &value); //replace selection or insert text at cursor
+
+  bool has_selection();
+
+
+  bool file_open (const QString &fileName, const QString &codec);
+  bool file_save_with_name (const QString &fileName, const QString &codec);
+  bool file_save_with_name_plain (const QString &fileName);
+
+  int get_tab_idx();
+  QString get_triplex();
+  QString get_filename_at_cursor();
+  QStringList get_words();
+
+  void goto_pos (int pos);
+
+  void set_tab_caption (const QString &fileName);
+  void set_hl (bool mode_auto = true, const QString &theext = "txt");
+  void set_markup_mode();
+
+  void insert_image (const QString &full_path);
+  void reload (const QString &enc);
+
+  void update_status();
+  void update_title (bool fullname = true);
+  void update_labels();
+
+  void set_show_linenums (bool enable);
+  void set_show_margin (bool enable);
+  void set_margin_pos (int mp);
+  void set_hl_cur_line (bool enable);
+  void set_hl_brackets (bool enable);
+
+  void set_word_wrap (bool wrap);
+  bool get_word_wrap();
+
+  void indent();
+  void un_indent();
+  void calc_auto_indent();
+
+  void setup_brace_width();
+  void brace_highlight();
+
+  void update_ext_selections();
+
+  void rect_block_start();
+  void rect_block_end();
+
+  bool has_rect_selection() const;
+  void rect_sel_reset();
+  void rect_sel_replace (const QString &s, bool insert = false);
+  void rect_sel_upd();
+  QString rect_sel_get() const;
+  void rect_sel_cut (bool just_del = false);
+
+  void lineNumberAreaPaintEvent (QPaintEvent *event);
+  int line_number_area_width();
+
+public slots:
+
+  void updateLineNumberAreaWidth();
+  void cb_cursorPositionChanged();
+  void updateLineNumberArea (const QRect &, int);
+  void slot_selectionChanged();
+
+  void ed_copy();
+  void ed_cut();
+  void ed_paste();
+
 
-  void create_new();
-  Q_INVOKABLE void set_tab_caption (const QString &fileName);
-  Q_INVOKABLE bool save_with_name (const QString &fileName, const QString &codec);
-  Q_INVOKABLE bool save_with_name_plain (const QString &fileName);
-  Q_INVOKABLE void goto_pos (int pos);
-  Q_INVOKABLE QString get_filename_at_cursor();
-  Q_INVOKABLE QStringList get_words();
-  Q_INVOKABLE void set_hl (bool mode_auto = true, const QString &theext = "txt");
-  Q_INVOKABLE bool open_file (const QString &fileName, const QString &codec);
-  Q_INVOKABLE void insert_image (const QString &full_path);
-  Q_INVOKABLE void set_markup_mode();
-  Q_INVOKABLE void reload (const QString &enc);
-  Q_INVOKABLE int get_tab_idx();
-  Q_INVOKABLE QString get_triplex();
-  Q_INVOKABLE void update_status();
-  Q_INVOKABLE void update_title (bool fullname = true);
-  Q_INVOKABLE void update_labels();
 };
 
 
@@ -285,44 +291,50 @@ class CDox: public QObject
 
 public:
 
+  QStringList recent_files;
+
+//regexp pattern and file name of syntax hl rules
+#if QT_VERSION >= 0x050000
+  std::vector<std::pair <QRegularExpression, QString> > hl_files;
+#else
+  std::vector<std::pair <QRegExp, QString> > hl_files;
+#endif
+
+  std::vector <CDocument*> items;
+
+  QHash <QString, QString> markup_modes;
+  QHash <QString, QString> hash_project;
+  QHash <QString, QString> autosave_files;
+
   CTioHandler tio_handler;
   CTodo todo;
 
-  QStringList recent_files;
-
+  QString dir_last;
   QString fname_current_session;
   QString fname_current_project;
   QString dir_config;
   QString fname_crapbook;
+  QString fname_saved_buffers;
+
   QString markup_mode;
   QString recent_list_fname;
 
-  QHash <QString, QString> hls;
-  QHash <QString, QString> markup_modes;
-  QHash <QString, QString> hash_project;
-
-  std::vector <CDocument*> items;
-
   QLabel *l_status_bar;
   QLabel *l_charset;
- // QStatusBar *status_bar;
-  CLogMemo *log;
-  QMainWindow *parent_wnd;
-  QTabWidget *tab_widget;
-  QTabWidget *main_tab_widget;
-  QMenu *recent_menu;
-  QTimer *timer;
+
+  CLogMemo *log; //uplink
+  QMainWindow *parent_wnd; //uplink
+  QTabWidget *tab_widget; //uplink
+  QTabWidget *main_tab_widget; //uplink
+  QMenu *menu_recent; //uplink
+  QTimer timer_joystick;
+  QTimer timer_autosave;
 
 #if defined(JOYSTICK_SUPPORTED)
-
   CJoystick *joystick;
-
-  bool event (QEvent *ev);
-  void handle_joystick_event (CJoystickAxisEvent *ev);
-
 #endif
 
-   CDox();
+  CDox();
   ~CDox();
 
   void update_project (const QString &fileName);
@@ -333,24 +345,37 @@ public:
 
   void move_cursor (QTextCursor::MoveOperation mo);
 
-  Q_INVOKABLE CDocument* create_new();
-  Q_INVOKABLE CDocument* open_file (const QString &fileName, const QString &codec);
-  Q_INVOKABLE CDocument* open_file_triplex (const QString &triplex);
-  Q_INVOKABLE CDocument* get_document_by_fname (const QString &fileName);
-  Q_INVOKABLE CDocument* get_current();
+  CDocument* create_new();
+  CDocument* open_file (const QString &fileName, const QString &codec);
+  CDocument* open_file_triplex (const QString &triplex);
+  CDocument* get_document_by_fname (const QString &fileName);
+  CDocument* get_current();
+
+  void close_by_idx (int i);
+  void close_current();
+
+  void save_to_session (const QString &fileName);
+  void load_from_session (const QString &fileName);
+
+  void save_buffers (const QString &fileName);
+  void load_from_buffers (const QString &fileName);
+
+  void apply_settings();
+  void apply_settings_single (CDocument *d);
+
+#if defined(JOYSTICK_SUPPORTED)
+  bool event (QEvent *ev);
+  void handle_joystick_event (CJoystickAxisEvent *ev);
+#endif
 
-  Q_INVOKABLE void close_by_idx (int i);
-  Q_INVOKABLE void close_current();
-  Q_INVOKABLE void save_to_session (const QString &fileName);
-  Q_INVOKABLE void load_from_session (const QString &fileName);
-  Q_INVOKABLE void apply_settings();
-  Q_INVOKABLE void apply_settings_single (CDocument *d);
 
 public slots:
 
   void open_recent();
   void open_current();
 
+  void autosave();
+
   void move_cursor_up();
   void move_cursor_down();
   void move_cursor_left();
@@ -364,22 +389,21 @@ class CLineNumberArea: public QWidget
 {
 public:
 
-  CTEAEdit *code_editor;
+  CDocument *code_editor; //uplink
 
-  CLineNumberArea (CTEAEdit *editor = 0): QWidget (editor), code_editor (editor) {}
+  CLineNumberArea (CDocument *editor = 0): QWidget (editor), code_editor (editor) {}
 
   QSize sizeHint() const {
-                          return QSize(code_editor->lineNumberAreaWidth(), 0);
+                          return QSize (code_editor->line_number_area_width(), 0);
                          }
 
 protected:
 
   void paintEvent (QPaintEvent *event)
-       {
-        code_editor->lineNumberAreaPaintEvent (event);
-       }
+      {
+       code_editor->lineNumberAreaPaintEvent (event);
+      }
 
 };
 
 #endif
-
