@@ -2,7 +2,6 @@
 this code is Public Domain
 */
 
-#include <QDir>
 #include <QFileInfoList>
 #include <QFileInfo>
 #include <QLabel>
@@ -15,6 +14,7 @@ this code is Public Domain
 #include "utils.h"
 
 
+
 void create_menu_from_list (QObject *handler,
                             QMenu *menu,
                             const QStringList &list,
@@ -27,14 +27,45 @@ void create_menu_from_list (QObject *handler,
       {
        if (! i->startsWith ("#"))
           {
-           QAction *act = new QAction (*i, menu->parentWidget());
+           QAction *act = menu->addAction (*i);
            handler->connect (act, SIGNAL(triggered()), handler, method);
-           menu->addAction (act);
           }
       }
 }
 
 
+//uses dir name as menuitem, no recursion
+void create_menu_from_themes (QObject *handler,
+                              QMenu *menu,
+                              const QString &dir,
+                              const char *method
+                              )
+{
+  menu->setTearOffEnabled (true);
+  QDir d (dir);
+  QFileInfoList lst_fi = d.entryInfoList (QDir::NoDotAndDotDot | QDir::Dirs,
+                                          QDir::IgnoreCase | QDir::LocaleAware | QDir::Name);
+
+  for (QList <QFileInfo>::iterator fi = lst_fi.begin(); fi != lst_fi.end(); ++fi)
+      {
+       if (fi->isDir())
+          {
+           if (has_css_file (fi->absoluteFilePath()))
+              {
+               QAction *act = menu->addAction (fi->fileName());
+               act->setData (fi->filePath());
+               handler->connect (act, SIGNAL(triggered()), handler, method);
+              }
+           else
+               {
+                QMenu *mni_temp = menu->addMenu (fi->fileName());
+                create_menu_from_themes (handler, mni_temp, fi->filePath(), method);
+               }
+           }
+       }
+}
+
+
 void create_menu_from_dir (QObject *handler,
                            QMenu *menu,
                            const QString &dir,
@@ -56,15 +87,13 @@ void create_menu_from_dir (QObject *handler,
        if (fi->isDir())
           {
            QMenu *mni_temp = menu->addMenu (fi->fileName());
-           create_menu_from_dir (handler, mni_temp,
-                                 fi->filePath(), method);
+           create_menu_from_dir (handler, mni_temp, fi->filePath(), method);
           }
        else
            {
-            QAction *act = new QAction (fi->fileName(), menu->parentWidget());
+            QAction *act = menu->addAction (fi->fileName());
             act->setData (fi->filePath());
             handler->connect (act, SIGNAL(triggered()), handler, method);
-            menu->addAction (act);
            }
       }
 }
@@ -177,4 +206,3 @@ QComboBox* new_combobox (QBoxLayout *layout,
 
   return r;
 }
-
