@@ -7,6 +7,7 @@
 #include <QSpinBox>
 #include <QComboBox>
 #include <QBoxLayout>
+#include <QDir>
 
 
 void create_menu_from_list (QObject *handler,
@@ -15,6 +16,13 @@ void create_menu_from_list (QObject *handler,
                             const char *method
                            );
 
+
+void create_menu_from_themes (QObject *handler,
+                              QMenu *menu,
+                              const QString &dir,
+                              const char *method
+                              );
+
 void create_menu_from_dir (QObject *handler,
                            QMenu *menu,
                            const QString &dir,
