@@ -56,7 +56,7 @@ double calculate (string expression)
 
 //open braces
 
-  size_t end_pos = expression.find (")"); 
+  size_t end_pos = expression.find (')');
   
 //  cout << " end pos = " << end_pos << endl;    
     
@@ -64,7 +64,6 @@ double calculate (string expression)
   if (end_pos != string::npos)
       do 
         {
-//         for (size_t i = end_pos; i >= 0; i--)
          for (size_t i = end_pos; i-- > 0 ;)
              {
               if (expression[i] == '(')
@@ -74,7 +73,6 @@ double calculate (string expression)
                  }
              }
      
-         //end_pos = expression.find (")"); 
          string s_temp_value = expression.substr (start_pos + 1, end_pos - start_pos - 1); 
  
         //cout << "start_pos = " << start_pos << " end pos = " << end_pos << endl;    
@@ -136,7 +134,6 @@ double calculate (string expression)
      }
 
 
-
   list<CItem>::iterator p = items.begin();
 
 //степень и процент
@@ -168,7 +165,7 @@ double calculate (string expression)
           continue;
          }
 
-      p++;
+      ++p;
      }
   while (p != items.end());
 
@@ -204,7 +201,7 @@ double calculate (string expression)
           continue;
          }
 
-      p++;
+      ++p;
      }
   while (p != items.end());
 
@@ -216,7 +213,7 @@ double calculate (string expression)
       CItem current = *p;
 
       list<CItem>::iterator t = p;
-      t++;
+      ++t;
 
       CItem next = *t;
 
@@ -239,7 +236,7 @@ double calculate (string expression)
            continue;
           }
 
-         p++;
+         ++p;
         }
   while (p != items.end());
 
@@ -248,4 +245,3 @@ double calculate (string expression)
 
   return item.val;
 }
-
