@@ -22,11 +22,13 @@ Original ZIP package is copyrighted by Gilles Vollant and contributors,
 see quazip/(un)zip.h files for details. Basically it's the zlib license.
 */
 
-#include "quazipdir.h"
+
+#include <algorithm>
 
 #include <QSet>
 #include <QSharedData>
-#include <algorithm>
+
+#include "quazipdir.h"
 
 
 /// \cond internal
@@ -105,7 +107,9 @@ bool QuaZipDir::cd(const QString &directoryName)
             if (!dir.cd("/"))
                 return false;
         }
-        QStringList path = dirName.split('/', QString::SkipEmptyParts);
+//        QStringList path = dirName.split('/', QString::SkipEmptyParts);
+        QStringList path = dirName.split('/');
+
         for (QStringList::const_iterator i = path.constBegin();
                 i != path.end();
                 ++i) {
