@@ -1,5 +1,5 @@
 /***************************************************************************
- *   2007-2019 by Peter Semiletov                                          *
+ *   2007-2021 by Peter Semiletov                                          *
  *   peter.semiletov@gmail.com                                             *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -18,48 +18,71 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
+
+#ifdef HUNSPELL_ENABLE
+
 #ifdef H_DEPRECATED
 #include <string>
 #include <vector>
 #endif
 
-#ifdef HUNSPELL_ENABLE
 #include <hunspell/hunspell.hxx>
+
 #endif
 
+
 #include <QTextCodec>
 #include <QDir>
-#include <QRegExp>
+#include <QMessageBox>
+#include <QObject>
 
 #include "spellchecker.h"
 #include "utils.h"
 
 
-
 #ifdef ASPELL_ENABLE
 
-CAspellchecker::CAspellchecker (const QString &lang, const QString &path,
-                                const QString &user_path):
-                                CSpellchecker (lang, path, user_path)
+QString aspell_default_dict_path()
 {
-  qDebug() << "CAspellchecker::CAspellchecker - start";
-  ret = 0;
-  speller = 0;
-  spell_config = 0;
+  QString r;
+  r = "/usr/lib/aspell-0.60";
 
-  initialized = false;
+#if defined(Q_OS_OS2)
+
+  r = qEnvironmentVariable ("unixroot", "c:") + "\\usr\\lib\\aspell-0.60";
+
+#endif
+
+#if defined(Q_OS_WIN)
+
+  r = "C:\\Program Files\\Aspell";
+
+#endif
+
+  return r;
+}
+
+
+void CAspellchecker::load_dict()
+{
+  loaded = false;
 
-  spell_config = new_aspell_config();
   if (! spell_config)
      return;
 
-  aspell_config_replace (spell_config, "lang", lang.toUtf8().data());
+  if (speller)
+     {
+      delete_aspell_speller (speller);
+      speller = 0;
+     }
+
+  aspell_config_replace (spell_config, "lang", language.toUtf8().data());
   aspell_config_replace (spell_config, "encoding", "UTF-8");
 
 #if defined(Q_OS_WIN) || defined(Q_OS_OS2)
 
-  QString data_path = path + "\\data";
-  QString dict_path = path + "\\dict";
+  QString data_path = dir_dicts + "\\data";
+  QString dict_path = dir_dicts + "\\dict";
 
   data_path = data_path.replace ("/", "\\");
   dict_path = dict_path.replace ("/", "\\");
@@ -81,7 +104,22 @@ CAspellchecker::CAspellchecker (const QString &lang, const QString &path,
      }
 
   speller = to_aspell_speller (ret);
-  initialized = true;
+
+  loaded = true;
+}
+
+
+CAspellchecker::CAspellchecker (const QString &lang, const QString &path,
+                                const QString &user_path):
+                                CSpellchecker (lang, path, user_path)
+{
+  ret = 0;
+  speller = 0;
+  spell_config = 0;
+
+  spell_config = new_aspell_config();
+  if (! spell_config)
+     return;
 }
 
 
@@ -97,7 +135,7 @@ CAspellchecker::~CAspellchecker()
 
 void CAspellchecker::add_to_user_dict (const QString &word)
 {
-  if (! initialized || ! speller)
+  if (! loaded || ! speller)
      return;
 
   if (word.isEmpty())
@@ -112,74 +150,61 @@ void CAspellchecker::add_to_user_dict (const QString &word)
 
 void CAspellchecker::remove_from_user_dict (const QString &word)
 {
-  //const AspellWordList *awl = aspell_speller_personal_word_list (speller);
+  //not implemented at aspell
 }
 
 
 void CAspellchecker::change_lang (const QString &lang)
 {
-  if (! spell_config)
+  if (lang.isEmpty() || language == lang)
      return;
 
-  if (lang.isEmpty())
-     return;
-
-  aspell_config_replace (spell_config, "lang", lang.toUtf8().data());
-  aspell_config_replace (spell_config, "encoding", "UTF-8");
-
-  if (speller)
-     delete_aspell_speller (speller);
-
-  ret = new_aspell_speller (spell_config);
-
-  if (aspell_error (ret) != 0)
-     {
-      //g_print ("Error: %s\n", aspell_error_message (ret));
-      delete_aspell_can_have_error (ret);
-      return;
-     }
-
-  speller = to_aspell_speller (ret);
-  initialized = true;
+  language = lang;
 }
 
 
-QStringList CAspellchecker::get_speller_modules_list()
+void CAspellchecker::get_speller_modules_list()
 {
-  QStringList l;
+   modules_list.clear();
 
-  if (! spell_config)
-    return l;
+   if (! spell_config)
+      return;
 
   AspellDictInfoList *dlist;
   AspellDictInfoEnumeration *dels;
   const AspellDictInfo *entry;
 
   dlist = get_aspell_dict_info_list (spell_config);
-
   dels = aspell_dict_info_list_elements (dlist);
 
   while ((entry = aspell_dict_info_enumeration_next (dels)) != 0)
         {
-         if (entry)
-            l.prepend (entry->name);
+//       if (entry)
+         modules_list.prepend (entry->name);
         }
 
   delete_aspell_dict_info_enumeration (dels);
-
-  return l;
 }
 
 
 bool CAspellchecker::check (const QString &word)
 {
-  if (! initialized || ! speller)
+  if (modules_list.size() == 0)
+     {
+      QMessageBox::about (0, "!", QObject::tr ("Please set up spell checker dictionaries at\n Options - Functions page"));
       return false;
+     }
 
   if (word.isEmpty())
      return false;
 
-   return aspell_speller_check (speller, word.toUtf8().data(), -1);
+  if (! loaded)
+     load_dict();
+
+  if (speller)
+     return aspell_speller_check (speller, word.toUtf8().data(), -1);
+  else
+      return false;
 }
 
 
@@ -187,7 +212,7 @@ QStringList CAspellchecker::get_suggestions_list (const QString &word)
 {
   QStringList l;
 
-  if (! initialized || word.isEmpty() || !speller)
+  if (! loaded || word.isEmpty() || !speller)
      return l;
 
   const AspellWordList *suggestions = aspell_speller_suggest (speller, word.toUtf8().data(), -1);
@@ -211,108 +236,14 @@ QStringList CAspellchecker::get_suggestions_list (const QString &word)
 
 #ifdef HUNSPELL_ENABLE
 
-CHunspellChecker::CHunspellChecker (const QString &lang, const QString &path, const QString &user_path): CSpellchecker (lang, path, user_path)
-{
-//  qDebug() << "CHunspellChecker::CHunspellChecker - start";
 
-  initialized = false;
-
-  dict_dir = path;
-  user_dir = user_path;
-  lng = lang;
-
-#if defined(Q_OS_WIN) || defined(Q_OS_OS2)
-
-  QString fname_aff = path + QDir::separator() + lng + ".aff";
-  QString fname_dict = path + QDir::separator() + lng + ".dic";
-  QString fname_userdict = user_path + QDir::separator() + lng + ".dic";
-  QString fname_userdict_pure;
-
-  fname_aff = fname_aff.replace ("/", "\\");
-  fname_dict = fname_dict.replace ("/", "\\");
-  fname_userdict = fname_userdict.replace ("/", "\\");
-  fname_userdict_pure = fname_userdict;
-
-#else
-
-  QString fname_aff = path + QDir::separator() + lng + ".aff";
-  QString fname_dict = path + QDir::separator() + lng + ".dic";
-  QString fname_userdict = user_path + QDir::separator() + lng + ".dic";
-
-#endif
-
-
-#if defined(Q_OS_WIN)
-
-  fname_aff = "\\\\?\\" + fname_aff;
-  fname_dict = "\\\\?\\" + fname_dict;
-  fname_userdict = "\\\\?\\" + fname_userdict;
-
-#endif
-
-//qDebug() << fname_aff;
-//qDebug() << fname_dict;
-//qDebug() << fname_userdict;
-
-  if (dir_exists (dict_dir))
-     initialized = true;
-
-  speller = new Hunspell (fname_aff.toUtf8().data(), fname_dict.toUtf8().data());
-
-#if !defined (H_DEPRECATED)
-  encoding = speller->get_dic_encoding();
-#else
-  str_encoding = speller->get_dict_encoding();
-#endif
-
-
-#if defined(Q_OS_UNIX)
-
-  if (initialized)
-  if (file_exists (fname_userdict))
-     {
-      speller->add_dic (fname_userdict.toUtf8().data());
-
-#if ! defined (H_DEPRECATED)
-      user_words = qstring_load (fname_userdict, encoding).split ("\n");
-#else
-      user_words = qstring_load (fname_userdict, str_encoding.data()).split ("\n");
-
-#endif
-      user_words.removeFirst();
-     }
-#else  //! UNIX
-
-  if (initialized)
-  if (file_exists (fname_userdict_pure))
-     {
-      speller->add_dic (fname_userdict.toUtf8().data());
-#if ! defined (H_DEPRECATED)
-      user_words = qstring_load (fname_userdict, encoding).split ("\n");
-#else
-      user_words = qstring_load (fname_userdict, str_encoding.data()).split ("\n");
-#endif
-
-      user_words.removeFirst();
-     }
-
-#endif
-
-  //qDebug() << "CHunspellChecker::CHunspellChecker - end";
-}
-
-
-CHunspellChecker::~CHunspellChecker()
+void CHunspellChecker::save_user_dict() //uses current language
 {
 #if defined(Q_OS_WIN) || defined(Q_OS_OS2)
-
-  QString filename = user_dir + QDir::separator() + lng + ".dic";
-  filename = filename.replace ("/", "\\");
-
+   QString filename = dir_user_dicts + QDir::separator() + language + ".dic";
+   filename = filename.replace ("/", "\\");
 #else
-
-  QString filename = user_dir + QDir::separator() + lng + ".dic";
-
+   QString filename = dir_user_dicts + QDir::separator() + language + ".dic";
 #endif
 
   if (user_words.size() > 0)
@@ -324,27 +255,25 @@ CHunspellChecker::~CHunspellChecker()
       qstring_save (filename, user_words.join ("\n"), str_encoding.data());
 #endif
      }
-
-  delete speller;
 }
 
 
-void CHunspellChecker::change_lang (const QString &lang)
+void CHunspellChecker::load_dict()
 {
-  delete speller;
+  loaded = false;
 
-  initialized = false;
-  lng = lang;
-  user_words.clear();
 
-  if (dir_exists (dict_dir))
-     initialized = true;
+  if (! dir_exists (dir_dicts) || language.isEmpty())
+     return;
+
+  if (speller)
+     delete speller;
 
 #if defined(Q_OS_WIN) || defined(Q_OS_OS2)
 
-  QString fname_aff = dict_dir + QDir::separator() + lng + ".aff";
-  QString fname_dict = dict_dir + QDir::separator() + lng + ".dic";
-  QString fname_userdict = user_dir + QDir::separator() + lng + ".dic";
+  QString fname_aff = dir_dicts + QDir::separator() + language + ".aff";
+  QString fname_dict = dir_dicts + QDir::separator() + language + ".dic";
+  QString fname_userdict = dir_user_dicts + QDir::separator() + language + ".dic";
   QString fname_userdict_pure;
 
   fname_aff = fname_aff.replace ("/", "\\");
@@ -354,23 +283,32 @@ void CHunspellChecker::change_lang (const QString &lang)
 
 #else
 
-  QString fname_aff = dict_dir + QDir::separator() + lng + ".aff";
-  QString fname_dict = dict_dir + QDir::separator() + lng + ".dic";
-  QString fname_userdict = user_dir + QDir::separator() + lng + ".dic";
+  QString fname_aff = dir_dicts + QDir::separator() + language + ".aff";
+  QString fname_dict = dir_dicts + QDir::separator() + language + ".dic";
+  QString fname_userdict = dir_user_dicts + QDir::separator() + language + ".dic";
+
+#endif
+
+#if defined(Q_OS_WIN)
+
+  fname_aff = "\\\\?\\" + fname_aff;
+  fname_dict = "\\\\?\\" + fname_dict;
+  fname_userdict = "\\\\?\\" + fname_userdict;
 
 #endif
 
   speller = new Hunspell (fname_aff.toUtf8().data(), fname_dict.toUtf8().data());
 
-#if ! defined (H_DEPRECATED)
+#if !defined (H_DEPRECATED)
   encoding = speller->get_dic_encoding();
 #else
   str_encoding = speller->get_dict_encoding();
 #endif
 
+
 #if defined(Q_OS_UNIX)
 
-  if (initialized && file_exists (fname_userdict))
+  if (file_exists (fname_userdict))
      {
       speller->add_dic (fname_userdict.toUtf8().data());
 
@@ -378,12 +316,12 @@ void CHunspellChecker::change_lang (const QString &lang)
       user_words = qstring_load (fname_userdict, encoding).split ("\n");
 #else
       user_words = qstring_load (fname_userdict, str_encoding.data()).split ("\n");
-
 #endif
-      user_words.removeFirst(); //зачем я это закомментировал раньше?
+      user_words.removeFirst();
      }
-#else //! UNIX
-  if (initialized && file_exists (fname_userdict_pure))
+#else  //! UNIX
+
+  if (file_exists (fname_userdict_pure))
      {
       speller->add_dic (fname_userdict.toUtf8().data());
 #if ! defined (H_DEPRECATED)
@@ -391,16 +329,43 @@ void CHunspellChecker::change_lang (const QString &lang)
 #else
       user_words = qstring_load (fname_userdict, str_encoding.data()).split ("\n");
 #endif
-      user_words.removeFirst(); //зачем я это закомментировал раньше?
+      user_words.removeFirst();
      }
 
 #endif
+
+  loaded = true;
+
+}
+
+
+CHunspellChecker::CHunspellChecker (const QString &lang, const QString &dir_path, const QString &dir_user): CSpellchecker (lang, dir_path, dir_user)
+{
+  speller = 0;
+  encoding = 0;
+}
+
+
+CHunspellChecker::~CHunspellChecker()
+{
+  delete speller;
+}
+
+
+void CHunspellChecker::change_lang (const QString &lang)
+{
+  if (language == lang)
+      return;
+
+  language = lang;
+  save_user_dict();
+  user_words.clear();
 }
 
 
 void CHunspellChecker::add_to_user_dict (const QString &word)
 {
-  if (! initialized || word.isEmpty())
+  if (! loaded || word.isEmpty())
      return;
 
 #if !defined (H_DEPRECATED)
@@ -412,13 +377,20 @@ void CHunspellChecker::add_to_user_dict (const QString &word)
   QByteArray es = codec->fromUnicode (word);
   speller->add (es.data());
   user_words.append (word);
+  save_user_dict();
 }
 
 
 bool CHunspellChecker::check (const QString &word)
 {
-  if (! initialized)
+  if (modules_list.size() == 0)
+     {
+      QMessageBox::about (0, "!", QObject::tr ("Please set up spell checker dictionaries at\n Options - Functions page"));
       return false;
+     }
+
+  if (! loaded)
+     load_dict();
 
 #if ! defined (H_DEPRECATED)
  QTextCodec *codec = QTextCodec::codecForName (encoding);
@@ -429,7 +401,6 @@ bool CHunspellChecker::check (const QString &word)
  QByteArray es = codec->fromUnicode (word);
 
 #ifndef H_DEPRECATED
-
    return speller->spell (es.constData());
   //return speller->spell (es.data()); //old way
 #else
@@ -440,7 +411,7 @@ bool CHunspellChecker::check (const QString &word)
 
 void CHunspellChecker::remove_from_user_dict (const QString &word)
 {
-  if (! initialized || word.isEmpty())
+  if (! loaded || word.isEmpty())
       return;
 
 #ifndef H_DEPRECATED
@@ -453,17 +424,20 @@ void CHunspellChecker::remove_from_user_dict (const QString &word)
   speller->remove (es.data());
   int i = user_words.indexOf (word);
   if (i != -1)
-     user_words.removeAt (i);
+     {
+      user_words.removeAt (i);
+      save_user_dict();
+     }
 }
 
 
-QStringList CHunspellChecker::get_speller_modules_list()
+void CHunspellChecker::get_speller_modules_list()
 {
-  QStringList sl;
+  modules_list.clear();
 
-  QDir dir (dict_dir);
+  QDir dir (dir_dicts);
   if (! dir.exists())
-     return sl;
+     return;
 
   QStringList filters;
 
@@ -474,10 +448,8 @@ QStringList CHunspellChecker::get_speller_modules_list()
 
   for (int i = 0; i < fil.size(); i++)
       {
-       sl.append (fil[i].baseName());
+       modules_list.append (fil[i].baseName());
       }
-
-  return sl;
 }
 
 
@@ -485,7 +457,7 @@ QStringList CHunspellChecker::get_suggestions_list (const QString &word)
 {
   QStringList sl;
 
-  if (! initialized || word.isEmpty())
+  if (! loaded || word.isEmpty())
      return sl;
 
 #if !defined (H_DEPRECATED)
@@ -519,4 +491,20 @@ QStringList CHunspellChecker::get_suggestions_list (const QString &word)
   return sl;
 }
 
+
+QString hunspell_default_dict_path()
+{
+  QString r;
+  r = "/usr/share/hunspell";
+
+#if defined(Q_OS_OS2)
+  r = qEnvironmentVariable ("unixroot", "c:") + "\\usr\\share\\myspell";
+#endif
+
+#if defined(Q_OS_WIN)
+#endif
+
+  return r;
+}
+
 #endif
