@@ -1,10 +1,8 @@
 #ifndef TIO_H
 #define TIO_H
 
-
 #include <vector>
 
-
 #include <QObject>
 #include <QList>
 #include <QByteArray>
@@ -17,6 +15,7 @@ class CSignaturesList: public QObject
   Q_OBJECT
 
 public:
+
   QString encname;
   QList <QByteArray> words;
 };
@@ -44,10 +43,10 @@ class CTio: public QObject
 public:
 
   bool ronly;
+
   QString data;
   QString charset;
   QString error_string;
-
   QString eol;
 
   QStringList extensions;
@@ -102,7 +101,7 @@ public:
   bool load (const QString &fname);
 };
 
-
+#if QT_VERSION >= 0x050000
 class CTioEpub: public CTioReadOnly
 {
   Q_OBJECT
@@ -112,8 +111,9 @@ public:
   CTioEpub();
   bool load (const QString &fname);
 };
+#endif
 
-
+/*
 class CTioODT: public CTioReadOnly
 {
   Q_OBJECT
@@ -123,7 +123,7 @@ public:
   CTioODT();
   bool load (const QString &fname);
 };
-
+*/
 
 class CTioABW: public CTioReadOnly
 {
@@ -158,9 +158,7 @@ public:
 };
 
 
-
 #if defined(POPPLER_ENABLE)
-
 class CTioPDF: public CTioReadOnly
 {
   Q_OBJECT
@@ -170,7 +168,6 @@ public:
   CTioPDF();
   bool load (const QString &fname);
 };
-
 #endif
 
 
@@ -184,7 +181,6 @@ public:
   CTioDJVU();
   bool load (const QString &fname);
 };
-
 #endif
 
 
@@ -205,5 +201,38 @@ public:
   QStringList get_supported_exts();
 };
 
-#endif // TIO_H
 
+/*
+class CCharsetConverter
+{
+  Q_OBJECT
+
+public:
+
+  static virtual QStringList get_charsets() = 0;
+
+  virtual ~CCharsetConverter() {};
+};
+*/
+/*
+class CIconvCharsetConverter: public QObject
+{
+  Q_OBJECT
+
+public:
+
+  QStringList charsets;
+
+  QStringList charsets_for_locale (const QString &loc);
+
+  void load_charsets();
+
+  QString to_utf16 (const QString &enc_from, char *inbuf, int len);
+ // char* from_utf_16 (const QString &enc_to, const QString &data);
+
+
+};
+*/
+
+
+#endif // TIO_H
