@@ -55,6 +55,26 @@ Do you want to save your changes?</source>
         <translation>%1 a été modifié.
 Voulez-vous sauvegardez vos changements ?</translation>
     </message>
+    <message>
+        <source>Copy</source>
+        <translation type="unfinished">Copier</translation>
+    </message>
+    <message>
+        <source>Cut</source>
+        <translation type="unfinished">Couper</translation>
+    </message>
+    <message>
+        <source>Paste</source>
+        <translation type="unfinished">Coller</translation>
+    </message>
+    <message>
+        <source>Undo</source>
+        <translation type="unfinished">Défaire</translation>
+    </message>
+    <message>
+        <source>Redo</source>
+        <translation type="unfinished">Refaire</translation>
+    </message>
 </context>
 <context>
     <name>CImgViewer</name>
@@ -85,10 +105,6 @@ Voulez-vous sauvegardez vos changements ?</translation>
         <source>famous input field</source>
         <translation></translation>
     </message>
-    <message>
-        <source>All (*);;Text files (*.txt);;Markup files (*.xml *.html *.htm *.);;C/C++ (*.c *.h *.cpp *.hh *.c++ *.h++ *.cxx)</source>
-        <translation></translation>
-    </message>
     <message>
         <source>Charset</source>
         <translation>Jeu de caractères</translation>
@@ -1830,6 +1846,18 @@ Pour utiliser des penses-bêtes horodatés, spécifiez l&apos;heure au format 24
         <source>Subtitles: shift timecodes by msecs</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Preview Markdown</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>All (*);;Text files (*.txt);;Markup files (*.xml *.html *.xhtml *.htm *.md);;C/C++ (*.c *.h *.cpp *.hh *.c++ *.h++ *.cxx)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Select all</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>CTioReadOnly</name>
