@@ -6,14 +6,17 @@ Peter Semiletov
 #ifndef UTILS_H
 #define UTILS_H
 
+#include <vector>
+
 #include <QObject>
 #include <QHash>
 #include <QFileInfo>
 #include <QStringList>
 
-
-#if QT_VERSION >= 0x050000
-#include <QMimeDatabase>
+#if QT_VERSION < 0x050000
+#include <QRegExp>
+#else
+#include <QRegularExpression>
 #endif
 
 
@@ -32,26 +35,28 @@ class CFTypeChecker: public QObject
 {
 public:
 
-  QStringList lexts;
-  QStringList lnames;
-
-#if QT_VERSION >= 0x050000
-  QMimeDatabase db;
+#if QT_VERSION < 0x050000
+  std::vector <QRegExp> patterns;
+#else
+  std::vector <QRegularExpression> patterns;
 #endif
 
-  CFTypeChecker (const QStringList &fnames, const QStringList &exts);
+  CFTypeChecker();
+
   bool check (const QString &fname) const; 
-  QStringList get_supported_exts() const;
 };
 
 
 /* file utils */
 
+bool has_css_file (const QString &path);
 QString guess_enc_for_file (const QString &fname);
 
-
 bool file_is_writable (const QString &fname);
 bool file_is_readable (const QString &fname);
+bool path_is_file (const QString &fname);
+bool path_is_dir (const QString &fname);
+bool path_is_abs (const QString &fname);
 
 bool dir_exists (const QString &path);
 bool file_exists (const QString &fileName);
@@ -69,10 +74,9 @@ QString qstring_load (const QString &fileName, const char *enc = "UTF-8");
 QString qstring_load_first_line (const QString &fileName);
 QByteArray file_load (const QString &fileName);
 
-
-
 /* string/stringlist utils */
 
+//bool str_check (char *s1, char *s2, int size);
 QString string_between (const QString &source, const QString &sep1, const QString &sep2);
 bool char_is_bad (const QChar &c);
 void qstring_list_print (const QStringList &l);
@@ -85,6 +89,7 @@ void strlist_swap (QStringList &l, int a, int b);
 QString hash_get_val (QHash<QString, QString> &h, const QString &key, const QString &def_val);
 QString qstring_load_value (const QString &fileName, const QString &key, const QString &def);
 QHash<QString, QString> hash_load_keyval (const QString &fname);
+void hash_save_keyval (const QString &fname, const QHash <QString, QString> &h);
 
 
 /* image utils */
@@ -128,8 +133,13 @@ inline bool is_dir (const QString &path)
 
 inline QString get_file_path (const QString &fileName)
 {
-  return QFileInfo (fileName).absolutePath();
+  return QFileInfo (fileName).path();
 }
 
-#endif
 
+/*
+GUI functions
+*/
+
+
+#endif
